;   File: Remoto.ASM
;    PIC: 16F877A @ 20Mhz
;   Data: 28/05/2003
; Autor: Renie
;
; Descrio:
; Teste para PIC acessar portas etc. com comandos vindo do PC
; 19200 Baud, sem paridade, 1 Stop bit @ 20Mhz XTAL
; TX apartir do PIC em C6 (do PIC p/ P.C.) via MAX232
; RX entrada do pic em C7 (do P.C. p/ PIC) via MAX232
;
; Comandos:
; 0     Reset PIC 
; 1     inibe mensagem de confirmao dos comandos (OK) 21/07/2003
; 2     Reativar confirmao dos comandos  24/07/2003  
;	
; S     Enviar valor p/ endereo + 2 bytes p/endereco (banco,offset) +  1 byte (valor)
; L     Ler valor do endereo + 2 bytes p/endereo (banco,offset) devolve 1 byte
; E     Eco + 1 byte (qtd) + qdt bytes, devolve qtd bytes recebidos   (max 48) 
; 
;
; Any other command causes the error indicator to be set on the device. (C2)
;
;
; C6 - RS232 do PIC p/o MAX232
; C7 - RS232 do MAX232 p/o PIC

	LIST p=16f877a,c=140
        errorlevel 1,-(305)
        include "p16f877a.inc"

Bank0   equ     0x00
Bank1   equ     0x01
LOOP    equ     0x24
TEMP    equ     0x25
VALUE   equ     0x26
COUNT   equ     0x27
Byte0  	equ     0x70  ; 70 a 7F - comum a todos os bancos 
Byte1  	equ     0x71
Byte2  	equ     0x72
Byte3   equ     0x73
Byte4   equ     0x74
;;;IRP     equ     0x07


Boot    equ     0x28    	; controle primeira vez
Conf    equ     0x29            ; controle de confirmao (OK) 21/07/2003

	org 	0
        clrf 	STATUS
        clrf    Boot
        clrf    Conf
        goto 	BootStart
        org 	4
ISR     goto 	5            	; Interupt
	org 	5



BootStart
        ;
        ; Configurar I/O ports
        ;
        clrf    STATUS
        movlw   b'00000000'
        movwf   PORTA
        movlw   b'00000000'
        movwf   PORTB
        movlw   b'01000000'
        movwf   PORTC
        movlw   b'00000000'
        movwf   PORTD
        movlw   b'00000000'
        movwf   PORTE

        banksel TRISA           ; Bank 1
        movlw 	b'11111111'     ; tudo input
        movwf   TRISA
        movlw   b'00000110'
        movwf   ADCON1          ; PortA input digital

        movlw   b'11111111'     ; tudo input
        movwf   TRISB
        movlw   b'10000000'     ; Desabilitar resistores pullup PORTB
        movwf   OPTION_REG

        movlw   b'11000000' 
		    		;  RC2 - RC5  output LED's
                                ;  RC6 - RC7  input(serial)
        movwf   TRISC

        movlw   b'00000000'     ; todos output
        movwf   TRISD

        movlw   b'11100000'     ; tudo output (3 bit port)
        movwf   TRISE

	banksel PORTC

        movlw   d'64'           ; 19.2K Baud 20MHz XTAL

        banksel TRISA           ;  Bank 1
;
; Configurar gerador de baud rate
;
        movwf   SPBRG           ; bank 1
        movlw   b'00100100'     ; BRGH = 1(High speed) assincrono
        movwf   TXSTA           ; no bank 1
        banksel RCSTA           ; Bank 0
        movlw   b'10010000'     ; recepo assincrona
        movwf   RCSTA           ; no bank 0
        movf    RCREG,W         ; limpar buffer RX bank 0
 ;       movf    RCREG,W 
 ;	movf    RCREG,W

;
; Principal
;

        movlw   mReady          ; MSG - Ready
	call    SendMessage
	bsf    	Boot, 0      	; desmarcar primeira vez

WaitCom
;;        call 	LimpaBufferRX
        call    RecLoop         ; espera e l RS232 - caracter volta em 'w'

; Identificar comando
	movwf   TEMP            ; Salva 'W' em TEMP
        sublw   0x53
	btfsc   STATUS, Z       ; Se ASCII = 83 'S'
        goto 	CmdSet          ; S - define  valor para endereo, recebe banco,offset e valor 
       
	movfw   TEMP            ; Recupera 'W' de TEMP
        sublw   0x4C
	btfsc   STATUS, Z       ; Se ASCII = 76 'L'
        goto 	CmdLoad         ; L - Ler contedo do endereo, recebe banco e offset,
				;     devolve 1 byte

	movfw   TEMP            ; Recupera 'W' de TEMP
        sublw   0x45
	btfsc   STATUS, Z       ; Se ASCII = 69 'E'
        goto 	CmdEco          ; E - Eco, recebe qtd de bytes (n) a ecoar e devidos n bytes,
				;     devolve os n bytes recebidos

	movfw   TEMP            ; Recupera 'W' de TEMP
        sublw   0x36
	btfsc   STATUS, Z       ; Se ASCII = 54 '6'
        goto 	CmdVersao       ; Comando 6 - Devolver descrio e verso

	movfw   TEMP            ; Recupera 'W' de TEMP         21/07/2003
        sublw   0x31                                          ;21/07/2003
	btfsc   STATUS, Z       ; Se ASCII = 49 '1'            21/07/2003 
        goto 	CmdNconf        ; Comando 1 - supender confirmao (OK)  21/07/2003

	movfw   TEMP            ; Recupera 'W' de TEMP         24/07/2003
        sublw   0x32                                          ;24/07/2003
	btfsc   STATUS, Z       ; Se ASCII = 50 '2'            24/07/2003 
        goto 	Cmdconf         ; Comando 2 - ativar confirmao (OK)  24/07/2003

NxtCmd1
        movf    TEMP, W         ; Recuperar W
        sublw   0x30
	btfsc   STATUS, Z       ; SE ASCII = 48 '0'
        goto 	BootStart       ; Comando 0 - Reset


;
; Comando desconhecido
;
        bsf     PORTC, 2        ; acender LED C2

        movlw   mError          ; MSG - Erro !
	call    SendMessage
	goto 	WaitCom        	; Volta a esperar dados


CmdNconf						;21/07/2003
; Comando 1 - supender confirmao (OK)			 21/07/2003
	bsf    Conf,0           ; no confirmar com OK	 21/07/2003
        goto   WaitCom					;21/07/2003

Cmdconf						    	;24/07/2003
; Comando 2 - reativar confirmao (OK)			 24/07/2003
	bcf    Conf,0           ;  confirmar com OK	 24/07/2003
        goto   WaitCom					;24/07/2003



CmdSet
; Definir valor para endereo informado (banco,offset, valor)
	call   RecLoop          ; receber banco do endereo
        movwf  Byte1            ; guarda banco em temp1 (comum a todos os bancos)
        call   RecLoop          ; recebe offset do endereo
        movwf  Byte2            ; guarda offset em temp2 (comum a todos os bancos)     
	call   RecLoop          ; recebe valor a ser movido p/ o endereo
        movwf  Byte3		; guarda valor em temp3 (comum a todos os bancos)

	;;;; para implementar: testar para no alterar a configurao da RS232
	;; no bloquear alterar baud rate, experimentar alterar dinamicamente  24/07/2003
    
	btfss  Byte1,0		;24/07/2003 
	goto   okseguro		;24/07/2003
	movfw  Byte2		;24/07/2003
        sublw  0x07		;24/07/2003
	btfss  STATUS, Z        ;24/07/2003  
	goto   okseguro		;24/07/2003
	movlw  b'11000000'	;24/07/2003
        iorwf  Byte3,f		; garante no alterar TRISC RS232 24/07/2003


okseguro    	; setar banco
 	bcf    STATUS,IRP       ; inicialmente forar banco 0
        movfw  Byte1
	btfsc  Byte1,1          ; endereamento indireto banco = bit 7 do status (IRP) e 
                                ;Bit 7 do FSR         
        bsf    STATUS,IRP   
	bcf    Byte2,7
	btfsc  Byte1,0
        bsf    Byte2,7 
	movfw  Byte2
	movwf  FSR              ; offset para endereamento indireto 
	movfw  Byte3            ; valor a ser movido 
        clrf   INDF
	iorwf  INDF             ; mover o valor usando endereamento indireto
        goto   ConfirmCmd 

CmdLoad
; Ler contedo do endereo informado (banco,offset)
	call   RecLoop          ; receber banco do endereo
        movwf  Byte1            ; guarda banco em temp1 (comum a todos os bancos)
        call   RecLoop          ; recebe offset do endereo
        movwf  Byte2            ; guarda offset em temp2 (comum a todos os bancos)     
       	; setar banco
 	bcf    STATUS,IRP       ; inicialmente forar banco 0
	bcf    Byte2,7
        movfw  Byte1
	btfsc  Byte1,1          ; endereamento indireto banco = bit 7 do status (IRP) e Bit 7 do FSR         
        bsf    STATUS,IRP   
	btfsc  Byte1,0
        bsf    Byte2,7 
	movfw  Byte2
	movwf  FSR              ; offset para endereamento indireto 
        clrw
	iorwf   INDF,w             ; valor a ser movido
        banksel PIR1  
	call   TxByte           ; mover o valor usando endereamento indireto
        goto   ConfirmCmd 


CmdEco
; Ecoar n bytes recebidos - guarda os bytes de 40h em diante, mximo 48 bytes at 6Fh
	banksel PIR1
	call   RecLoop          ; receber quantidade de bytes a guardar
        movwf  Byte1            ; guarda qtd em Byte1 (comum a todos os bancos)
        sublw  0x30		; se qtd maior que 48, qtd=48
	btfsc  STATUS,C
	goto   pulo1
        movlw  0x30
        movwf  Byte1
pulo1   movfw  Byte1
	movwf  Byte2            ; contador qtd 
        movlw  0x40
        movwf  FSR
Prox    call   RecLoop
        movwf  INDF
        incf   FSR,F
        decf   Byte2
        btfss  STATUS,Z
        goto   Prox
		; transmitir os bytes recebidos
        movfw  Byte1
        movwf  Byte2
  	movlw  0x40
	movwf  FSR
Prox2	movfw  INDF
        call   TxByte
	incf   FSR,F
        decf   Byte2
	btfss  STATUS,Z
	goto   Prox2
        goto   ConfirmCmd 

CmdVersao
; Retorna nome prog. e verso
        movlw   mProduct
        call    SendMessage
        movlw   mVersion
        call 	SendMessage
        goto    WaitCom         ; Espera prximo comando






ConfirmCmd
        btfsc   Conf,0					;21/07/2003
        goto    Backloop        ; no confirmar com OK   21/07/2003
        movlw   mOK            	; MSG - OK.         
	call    SendMessage
Backloop
        btfsc   Conf,0          ; se no confirmar, no limpar buffer (perde prox dados) 21/07/2003
  ;;      call    LimpaBufferRX
	goto 	WaitCom        	; Espera prximo comando




;
; Recebe byte da RS232
; S volta da rotina depois de receber alguma coisa
;
RecLoop
        nop
        btfss   PIR1,RCIF       ; Checar por qualquer dado em RX
        goto 	RecLoop         ; Continuar at chegar algo em RX
        movf    RCREG,W         ; Passa o dado de RX para W
        return

;
; Envia byte para USART
;
TxByte
        nop
        btfss   PIR1,TXIF       ; Buffer TX vazio ?
        goto    TxByte          ; No - continua esperando

        movwf   TXREG           ; Buffer TX vazio - enviar o caracter
        return

LimpaBufferRX
	movf    RCREG,W        	; limpar buffer RX no banco 0
        movf    RCREG,W
	movf    RCREG,W
        return



;
; Enviar mensagem para serial
;
SendMessage
        movwf	LOOP            ; define o OFFSET da tabela
SendMes2
        movf    LOOP, W         ; Carrega ponteiro para caracteres
        call    MsgTable        ; Retona 'W' = caracter ASCII para enviar ou ASCII 0 = EOM
        movwf   TEMP            ; Salva o caracter recebido
        ; Testa EOM
        sublw   0x00

	btfsc   STATUS, Z    	; EOM encontrado?
        return                  ; sim

        movf    TEMP, W         ; Recupera valor original
	call    TxByte          ; Envia o caracter para serial
        incf    LOOP, F         ; Ponteirar prximo caracter
        goto    SendMes2        ; continua at acabar


;******************************************************************


;
; Tabela de mensagens
;
MsgTable
mError	equ     0x00
        addwf   PCL, F
        retlw   'E'             ; Erro
        retlw   'r'
        retlw   'r'
        retlw   'o'
        retlw   ' '
        retlw   '!'
	retlw   d'13'
        retlw   d'10'
        retlw   d'0'            ; EOM

mOK   	equ     0x09
        retlw   'O'             ; OK
        retlw   'K'
        retlw   '.'
        retlw   d'13'
        retlw   d'10'
        retlw   d'0'            ; EOM

mReady  equ     0x0F
        retlw   'R'             ; Ready
        retlw   'e'
        retlw   'a'
        retlw   'd'
        retlw   'y'
        retlw   '.'
        retlw   d'13'
        retlw   d'10'
        retlw   d'0'            ; EOM

mCRLF   equ     0x18            ; CRLF
        retlw   d'13'
        retlw   d'10'
        retlw   d'0'            ; EOM

mProduct equ    0x1B            ; Descrio
        retlw   'R'
        retlw   'e'
        retlw   'm'
        retlw   'o'
        retlw   't'
        retlw   'o'
        retlw   ' '
        retlw   'S'
        retlw   'e'
        retlw   'r'
        retlw   'i'
        retlw   'a'
	retlw   'l'
        retlw   ' '
        retlw   'P'
        retlw   'I'
        retlw   'C'
        retlw   d'13'
        retlw   d'10'
        retlw   d'0'            ; EOM

mVersion equ    0x2F            ; Nr Verso
        retlw   '1'
        retlw   '.'
        retlw   '1'
        retlw   '/'
        retlw   '0'
        retlw   '3'
        retlw   '.'
        retlw   d'13'
        retlw   d'10'
        retlw   d'0'            ; EOM

        end
